@@ -315,6 +315,12 @@ void Decoration::init()
     }
     m_item = qobject_cast< QQuickItem* >(component->create(context));
     if (!m_item) {
+        if (component->isError()) {
+            const auto errors = component->errors();
+            for (const auto &error: errors) {
+                qCWarning(AURORAE) << error;
+            }
+        }
         return;
     }
     m_item->setParent(this);
@@ -344,12 +350,6 @@ void Decoration::init()
 
         }
 
-        //workaround for https://codereview.qt-project.org/#/c/207198/
-#if (QT_VERSION < QT_VERSION_CHECK(5, 10, 0))
-        if (!usingGL) {
-            m_renderControl->sync();
-        }
-#endif
         // delay rendering a little bit for better performance
         m_updateTimer.reset(new QTimer);
         m_updateTimer->setSingleShot(true);
