@@ -38,6 +38,10 @@ class KWIN_EXPORT ScriptedEffect : public KWin::AnimationEffect
     Q_ENUMS(Anchor)
     Q_ENUMS(MetaType)
     Q_ENUMS(EasingCurve)
+    /**
+     * True if we are the active fullscreen effect
+     */
+    Q_PROPERTY(bool isActiveFullScreenEffect READ isActiveFullScreenEffect NOTIFY isActiveFullScreenEffectChanged)
 public:
     // copied from kwineffects.h
     enum DataRole {
@@ -76,6 +80,29 @@ public:
      * @returns @c true if another window has grabbed the effect, @c false otherwise
      **/
     Q_SCRIPTABLE bool isGrabbed(KWin::EffectWindow *w, DataRole grabRole);
+
+    /**
+     * Grabs the window with the specified role.
+     *
+     * @param w The window.
+     * @param grabRole The grab role.
+     * @param force By default, if the window is already grabbed by another effect,
+     *   then that window won't be grabbed by effect that called this method. If you
+     *   would like to grab a window even if it's grabbed by another effect, then
+     *   pass @c true.
+     * @returns @c true if the window was grabbed successfully, otherwise @c false.
+     **/
+    Q_SCRIPTABLE bool grab(KWin::EffectWindow *w, DataRole grabRole, bool force = false);
+
+    /**
+     * Ungrabs the window with the specified role.
+     *
+     * @param w The window.
+     * @param grabRole The grab role.
+     * @returns @c true if the window was ungrabbed successfully, otherwise @c false.
+     **/
+    Q_SCRIPTABLE bool ungrab(KWin::EffectWindow *w, DataRole grabRole);
+
     /**
      * Reads the value from the configuration data for the given key.
      * @param key The key to search for
@@ -91,14 +118,18 @@ public:
         return m_screenEdgeCallbacks;
     }
 
+    bool isActiveFullScreenEffect() const;
+
     bool registerTouchScreenCallback(int edge, QScriptValue callback);
     bool unregisterTouchScreenCallback(int edge);
 
 public Q_SLOTS:
     //curve should be of type QEasingCurve::type or ScriptedEffect::EasingCurve
-    quint64 animate(KWin::EffectWindow *w, Attribute a, int ms, KWin::FPx2 to, KWin::FPx2 from = KWin::FPx2(), uint metaData = 0, int curve = QEasingCurve::Linear, int delay = 0);
-    quint64 set(KWin::EffectWindow *w, Attribute a, int ms, KWin::FPx2 to, KWin::FPx2 from = KWin::FPx2(), uint metaData = 0, int curve = QEasingCurve::Linear, int delay = 0);
+    quint64 animate(KWin::EffectWindow *w, Attribute a, int ms, KWin::FPx2 to, KWin::FPx2 from = KWin::FPx2(), uint metaData = 0, int curve = QEasingCurve::Linear, int delay = 0, bool fullScreen = false, bool keepAlive = true);
+    quint64 set(KWin::EffectWindow *w, Attribute a, int ms, KWin::FPx2 to, KWin::FPx2 from = KWin::FPx2(), uint metaData = 0, int curve = QEasingCurve::Linear, int delay = 0, bool fullScreen = false, bool keepAlive = true);
     bool retarget(quint64 animationId, KWin::FPx2 newTarget, int newRemainingTime = -1);
+    bool redirect(quint64 animationId, Direction direction, TerminationFlags terminationFlags = TerminateAtSource);
+    bool complete(quint64 animationId);
     bool cancel(quint64 animationId) { return AnimationEffect::cancel(animationId); }
     virtual bool borderActivated(ElectricBorder border);
 
@@ -108,6 +139,7 @@ Q_SIGNALS:
      **/
     void configChanged();
     void animationEnded(KWin::EffectWindow *w, quint64 animationId);
+    void isActiveFullScreenEffectChanged();
 
 protected:
     ScriptedEffect();
@@ -127,6 +159,7 @@ private:
     KConfigLoader *m_config;
     int m_chainPosition;
     QHash<int, QAction*> m_touchScreenEdgeCallbacks;
+    Effect *m_activeFullScreenEffect = nullptr;
 };
 
 }
