@@ -195,6 +195,46 @@ QScriptValue unregisterScreenEdge(QScriptContext *context, QScriptEngine *engine
     return engine->newVariant(true);
 }
 
+template<class T>
+QScriptValue registerTouchScreenEdge(QScriptContext *context, QScriptEngine *engine)
+{
+    auto script = qobject_cast<T>(context->callee().data().toQObject());
+    if (!script) {
+        return engine->undefinedValue();
+    }
+    if (!validateParameters(context, 2, 2)) {
+        return engine->undefinedValue();
+    }
+    if (!validateArgumentType<int>(context)) {
+        return engine->undefinedValue();
+    }
+    if (!context->argument(1).isFunction()) {
+        context->throwError(QScriptContext::SyntaxError, i18nc("KWin Scripting error thrown due to incorrect argument",
+                                                               "Second argument to registerTouchScreenEdge needs to be a callback"));
+    }
+    const int edge = context->argument(0).toVariant().toInt();
+    const auto ret = script->registerTouchScreenCallback(edge, context->argument(1));
+    return engine->newVariant(ret);
+}
+
+template<class T>
+QScriptValue unregisterTouchScreenEdge(QScriptContext *context, QScriptEngine *engine)
+{
+    auto script = qobject_cast<T>(context->callee().data().toQObject());
+    if (!script) {
+        return engine->undefinedValue();
+    }
+    if (!validateParameters(context, 1, 1)) {
+        return engine->undefinedValue();
+    }
+    if (!validateArgumentType<int>(context)) {
+        return engine->undefinedValue();
+    }
+    const int edge = context->argument(0).toVariant().toInt();
+    const auto ret = script->unregisterTouchScreenCallback(edge);
+    return engine->newVariant(ret);
+}
+
 template<class T>
 QScriptValue registerUserActionsMenu(QScriptContext *context, QScriptEngine *engine)
 {
@@ -303,6 +343,20 @@ inline void unregisterScreenEdgeFunction(QObject *parent, QScriptEngine *engine,
     engine->globalObject().setProperty(QStringLiteral("unregisterScreenEdge"), shortcutFunc);
 }
 
+inline void registerTouchScreenEdgeFunction(QObject *parent, QScriptEngine *engine, QScriptEngine::FunctionSignature function)
+{
+    QScriptValue touchScreenFunc = engine->newFunction(function);
+    touchScreenFunc.setData(engine->newQObject(parent));
+    engine->globalObject().setProperty(QStringLiteral("registerTouchScreenEdge"), touchScreenFunc);
+}
+
+inline void unregisterTouchScreenEdgeFunction(QObject *parent, QScriptEngine *engine, QScriptEngine::FunctionSignature function)
+{
+    QScriptValue touchScreenFunc = engine->newFunction(function);
+    touchScreenFunc.setData(engine->newQObject(parent));
+    engine->globalObject().setProperty(QStringLiteral("unregisterTouchScreenEdge"), touchScreenFunc);
+}
+
 inline void registerUserActionsMenuFunction(QObject *parent, QScriptEngine *engine, QScriptEngine::FunctionSignature function)
 {
     QScriptValue shortcutFunc = engine->newFunction(function);
